//	Copyright (c) 2008 itok ( http://itok.jp/ , http://110k.net/ )
//	All rights reserved.
//
//	Redistribution and use in source and binary forms, with or without modification, 
//	are permitted provided that the following conditions are met:
//
//	- Redistributions of source code must retain the above copyright notice, 
//	  this list of conditions and the following disclaimer.
//	- Redistributions in binary form must reproduce the above copyright notice, 
//	  this list of conditions and the following disclaimer in the documentation 
//	  and/or other materials provided with the distribution.
//	- Neither the name of itok nor the names of its contributors may be used to endorse 
//	  or promote products derived from this software without specific prior written permission.
//
//	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS 
//	AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, 
//	INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
//	AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
//	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
//	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
//	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
//	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
//	HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
//	STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
//	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#import "NSXMLElement-ex.h"


@implementation NSXMLElement (PhotoUploader)

-(id) nodeForXPath:(NSString*)xpath error:(NSError**)error
{
	NSArray* nodes = [self nodesForXPath:xpath error:error];
	if (!nodes || [nodes count] == 0) {
		return nil;
	}
	if ([nodes count] == 1) {
		return [nodes objectAtIndex:0];
	}
	return nil;
}

-(NSArray*) attributesForXPath:(NSString*)xpath error:(NSError**)error
{
	NSXMLNode* node = [self nodeForXPath:xpath error:error];
	if (node) {
		if ([node kind] == NSXMLElementKind) {
			return [(NSXMLElement*)node attributes];
		}
	}
	return nil;
}

-(id) attributeValueForXPath:(NSString*)xpath andName:(NSString*)name error:(NSError**)error
{
	NSXMLNode* node = [self nodeForXPath:xpath error:error];
	if (node) {
		if ([node kind] == NSXMLElementKind) {
			return [(NSXMLElement*)node attributeValueForName:name]; 
			//[[(NSXMLElement*)node attributeForName:name] objectValue];
		}
	}
	return nil;
}

-(NSString*) objectValueForXPath:(NSString*)xpath error:(NSError**)error
{
	NSXMLNode* node = [self nodeForXPath:xpath error:error];
	if (node) {
		return [node objectValue];
	}
	return nil;
}

-(NSString*) attributeValueForName:(NSString*)name
{
	return [[self attributeForName:name] objectValue];
}

-(void) setAttribute:(NSXMLNode*)attribute
{
	[self removeAttributeForName:[attribute name]];
	[self addAttribute:attribute];
}

-(void) setAttributeValue:(NSString*)value forName:(NSString*)name
{
	NSXMLNode* attr = [[NSXMLNode alloc] initWithKind:NSXMLAttributeKind];
	[attr setObjectValue:value];
	[attr setName:name];
	[self setAttribute:attr];
	[attr release];
}


@end
